#!/bin/bash
set -e

dir=../experiments
images="7ca-33-x209y466.pgm 7ca-33-x362y289.pgm"
page_size=4096
box_size=$((page_size / 12))

mkdir -p $dir/images $dir/tmp

for image in $images; do
    ln -f ../images/$image $dir/images/
done

page_sizes="4096 6144 8192 12288 16384 32768 40960 49152 65536"

# Starting point: multi-box datasets with no perturbation
for image in $images; do
    for page_size in $page_sizes; do
	echo '*****' $image $page_size
	box_size=$((page_size / 12))
	# New way
	./imagetuples.py $dir/images/$image > $dir/tmp/tuples.$$
	./partition.sh $dir/tmp/tuples.$$ $dir/tmp/toconvert.$$ $page_size
	stem=`basename $image .pgm`-${page_size}
	./convertrtree "$dir/$stem" $box_size < $dir/tmp/toconvert.$$
	mv $dir/tmp/{tuples,toconvert}.$$ $dir/$stem/
    done
done

factors="8 16 32 64 128 256 512"
factors=512

# Many sizes and page sizes
for image in $images; do
    for page_size in $page_sizes; do
	echo '*****' $image $page_size
	stem=`basename $image .pgm`-$page_size
	for factor in $factors; do
	    ./perturb $dir/$stem $dir/${factor}x${stem} $factor
	done
    done
done

# One-box sets of varying numbers of points
for image in $images; do
    stem=`basename $image .pgm`
    for factor in $factors; do
	echo '*****' $image ${factor}x
	./large1 $dir/${factor}x${stem}-32768 $dir/large1-${factor}x${stem}/
    done
done

