#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <sysexits.h>
#include <assert.h>
#include <string>
#include <vlcutils/error.h>
#include <vlcutils/readline.h>
#include "ptopk.h"
#include "index.h"

int main(int argc, char *argv[])
{
     set_program_name(argv[0]);
     if (argc != 3) {
          fprintf(stderr, "Usage: %s INDEX-FILENAME BOX-SIZE\n", program_name);
          fprintf(stderr, "Append an object to a index structure.\n");
          exit(EX_USAGE);
     }
     std::string stem = argv[1];
     int box_size = atoi(argv[2]);

     WritableIndex index(stem, box_size, supersede);

     srandom(time(NULL));
     const char *line;
     while ((line = readline(stdin))) {
          int npoints = atoi(line);
          assert(npoints <= box_size);
          for (int i = 0; i < npoints; i++) {
               line = readline(stdin);
               double a[3];
               if (parseline_double_array(line, a, 3) != READLINE_OK)
                    fatal_error("Readline failed: %s", line);
               a[0] = a[0];
               index.addPoint(Point(a[0], a[1], a[2]));
          }
          index.commitBox();
     }
     index.commitObject();
     return EX_OK;
}
