/* genuniformdata.c: Generate a uniform dataset.

   Box coordinates are drawn from uniform distributions.  The boxes
   are then filled with points uniformly distributed within their
   boundaries. */

#define _GNU_SOURCE
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <sysexits.h>
#include <vlcutils/error.h>
#include "ptopk.h"

float random_float(float min, float max)
{
     return random() * (max - min) / RAND_MAX + min;
}

void random_page(struct box *box, struct point *points)
{
     for (;;) {
          box->p_min = 1.0 * random() / RAND_MAX;
          box->p_max = 1.0 * random() / RAND_MAX;
          box->x1_min = 1.0 * random() / RAND_MAX;
          box->x1_max = 1.0 * random() / RAND_MAX;
          box->x2_min = 1.0 * random() / RAND_MAX;
          box->x2_max = 1.0 * random() / RAND_MAX;
          if (box->p_max - box->p_min > 0.3)
               continue;
          if (box->x1_max - box->x1_min > 0.3)
               continue;
          if (box->x2_max - box->x2_min > 0.3)
               continue;
          if (box->p_min <= box->p_max && box->x1_min <= box->x1_max && 
               box->x2_min <= box->x2_max)
               break;
     }
     for (int j = 0; j < PAGE_SIZE; j++) {
          points[j].p = random_float(box->p_min, box->p_max);
          points[j].x1 = random_float(box->x1_min, box->x1_max);
          points[j].x2 = random_float(box->x2_min, box->x2_max);
     }
}

int main(int argc, char *argv[])
{
     set_program_name(argv[0]);
     if (argc != 3) {
          fprintf(stderr, "Usage: %s FILENAME-STEM N\n", program_name);
          exit(EX_USAGE);
     }
     char *stem = argv[1];
     int npages = atoi(argv[2]);
     if (npages <= 0)
          fatal_error("Number of pages must be positive: %s", argv[2]);

     char *points_filename, *boxes_filename;
     if (asprintf(&boxes_filename, "%s.boxes", stem) == -1)
          fatal_perror("%s", stem);
     FILE *boxes_file = fopen(boxes_filename, "w");
     if (!boxes_file) fatal_perror("%s", boxes_filename);
     if (asprintf(&points_filename, "%s.points", stem) == -1)
          fatal_perror("%s", stem);
     FILE *points_file = fopen(points_filename, "w");
     if (!points_file) fatal_perror("%s", points_filename);

     srandom(time(NULL));
     for (int i = 0; i < npages; i++) {
          struct box box;
          struct point points[BOX_SIZE];
          random_page(&box, points);
          fwrite(&box, sizeof(struct box), 1, boxes_file);
          fwrite(points, sizeof(struct point), BOX_SIZE, points_file);
     }
     fclose(boxes_file);
     fclose(points_file);
     return EX_OK;
}
