#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <vlcutils/mem.h>
#define HISTOGRAM_C
#include "histogram.h"

struct histogram *alloc_histogram(int nbins)
{
     struct histogram *h = mallock(sizeof(struct histogram) + 
                                   (nbins + 1) * sizeof(int));
     h->nbins = nbins;
     h->nsamples = 0;
     h->bins = (int *)(h + 1);
     memset(h->bins, 0, (nbins + 1) * sizeof(int));
     return h;
}

struct histogram *histogram(float *samples, int nsamples)
{
     return histogram_nbins(samples, nsamples, 100);
}

struct histogram *histogram_nbins(float *samples, int nsamples, int nbins)
{
     float min = samples[0];
     float max = samples[0];
     for (int i = 1; i < nsamples; i++) {
          if (samples[i] < min) min = samples[i];
          if (samples[i] > max) max = samples[i];
     }
     struct histogram *h = alloc_histogram(nbins);
     h->min = min;
     h->max = max;
     float bin_size = (max - min) / nbins;
     for (int i = 0; i < nsamples; i++) {
          assert(samples[i] >= min && samples[i] <= max);
          int bin = floor((samples[i] - min) / (bin_size == 0 ? 1 : bin_size));
          if (bin == nbins) {
               assert(samples[i] == max);
               bin--;
          }
          assert(bin >= 0 && bin <= nbins);
          h->bins[bin]++;
          h->nsamples++;
     }
     /* Now convert from pdf to cdf. */
     for (int i = 0; i < nbins; i++)
          h->bins[i + 1] += h->bins[i];
     for (int i = nbins; i >= 1; i--)
          h->bins[i] = h->bins[i - 1];
     h->bins[0] = 0;

#ifndef NDEBUG
     // Exercise the pvalue function in order to test it.
     for (int i = 0; i < nsamples; i++) {
          histogram_pvalue(h, samples[i]);
          if (i > 0)
               histogram_pvalue(h, (samples[i - 1] + samples[i]) / 2);
     }
#endif

     return h;
}

float pdf_x(const struct histogram *h, int i)
{
     assert(i >= 0 && i < h->nbins);
     float bin_size = (h->max - h->min) / h->nbins;
     return h->min + (i + 0.5) * bin_size;
}

float pdf_z(const struct histogram *h, int i)
{
     assert(i >= 0 && i < h->nbins);
     return (h->bins[i + 1] - h->bins[i]) * 1.0 / h->nsamples;
}

float cdf_z(const struct histogram *h, int i)
{
     assert(i >= 0 && i <= h->nbins);
     return 1.0 * h->bins[i] / h->nsamples;
}

void plot_pdf(const struct histogram *h)
{
     for (int i = 0; i < h->nbins; i++)
          printf("%f %f\n", pdf_x(h, i), pdf_z(h, i));
     printf("\n\n");
}

void plot_cdf(const struct histogram *h)
{
     float bin_size = (h->max - h->min) / h->nbins;
     for (int i = 0; i <= h->nbins; i++)
          printf("%f %f\n", cdf_x(h, i), cdf_z(h, i));
}
