#ifndef HISTOGRAM_H
#define HISTOGRAM_H

#ifdef __cplusplus
extern "C" {
#endif

#include <assert.h>
#include <math.h>

struct histogram {
     int nbins;
     int nsamples;
     float min, max;
     int *bins;
};

struct histogram *histogram(float *samples, int nsamples);
struct histogram *histogram_nbins(float *samples, int nsamples, int nbins);

float pdf_x(const struct histogram *h, int i);
float pdf_z(const struct histogram *h, int i);
float cdf_x(const struct histogram *h, int i);
float cdf_z(const struct histogram *h, int i);

void plot_pdf(const struct histogram *h);
void plot_cdf(const struct histogram *h);

#ifdef HISTOGRAM_C
#define INLINE
#else
#define INLINE extern inline
#endif

INLINE float cdf_x(const struct histogram *h, int i)
{
     assert(i >= 0 && i <= h->nbins);
     float bin_size = (h->max - h->min) / h->nbins;
     return h->min + i * bin_size;
}

INLINE float histogram_pvalue(const struct histogram *h, float x) {
     if (x < h->min) return 0.0;
     if (x >= h->max) return 1.0;
     int i = (int)round((x - h->min) * h->nbins / (h->max - h->min));
     assert(i >= 0 && i <= h->nbins);
     float pv = cdf_z(h, i);
     assert(pv >= 0 && pv <= 1);
     return pv;
}

INLINE float histogram_pvalue_min(const struct histogram *h, float x) {
     if (x < h->min) return 0.0;
     if (x >= h->max) return 1.0;
     int i = (int)floor((x - h->min) * h->nbins / (h->max - h->min));
     assert(i >= 0 && i <= h->nbins);
     float pv = cdf_z(h, i);
     assert(pv >= 0 && pv <= 1);
     return pv;
}

INLINE float histogram_pvalue_max(const struct histogram *h, float x) {
     if (x < h->min) return 0.0;
     if (x >= h->max) return 1.0;
     int i = (int)ceil((x - h->min) * h->nbins / (h->max - h->min));
     assert(i >= 0 && i <= h->nbins);
     float pv = cdf_z(h, i);
     assert(pv >= 0 && pv <= 1);
     return pv;
}

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
