#ifndef INDEX_H
#define INDEX_H

#include <string>
#include "mmapping.h"
#include "ptopk.h"

struct point {
     float p;
     float x1, x2;
};

struct box {
     float p_min, p_max;
     float x1_min, x1_max;
     float x2_min, x2_max;
};

typedef enum { fail, supersede, append } ifexists_t;

class ReadableIndex : public BoxSource {
     Mmapping<int> *objects;
     Mmapping<struct box> *boxes;
     Mmapping<struct point> *points;
     int npoints_cached;
     int *points_per_box;
public:
     const std::string filename;
     const int nobjects;
     ReadableIndex(const std::string filename);
     ~ReadableIndex();
     void info();
     Box get_box(int page_id);
     virtual int object_id(int page_id) const;
     int nboxes() const;
     int nboxes(int oid) const;
     double average_volume();
     // Linear access to points.
     int npoints();
     Point get_point(int i);
};

class WritableIndex {
     FILE *of, *bf, *pf;
     int nobjects; // Number of the current object.
     int nboxes; // Number of current box in this object.
     int npoints; // Number of points in current box.
     struct box box;
     struct point *points;
     int box_size;
public:
     WritableIndex(std::string filename, int box_size, ifexists_t ifexists);
     ~WritableIndex();
     void addPoint(Point p);
     void commitBox();
     void commitObject();
};


#endif
