#ifndef JOIN_H
#define JOIN_H

#include <list>
#include "ptopk.h"

class Result {
 public:
     const int object_id_a, object_id_b;
     const Point a, b;
     float score;
     Result(int object_id_a, Point a, int object_id_b, Point b, float score);
     bool operator!=(const Result& other) const;
     void print();
};

class Results {
 public:
     virtual bool insert(Result r) = 0;
     double threshold;
     Results(double threshold) : threshold(threshold) {}
};


class TopKResults;

class TopKResults : public Results {
 protected:
     std::list<Result> results;
     void report_stats();
 public:
     int k;
     int iterations;
     int size();
     bool verbose;
     TopKResults(int k);
     bool has_threshold();
     // Insert returns true iff the list of results (and possibly the
     // threshold) changed.
     virtual bool insert(Result result);
     void print();
};

class TopKResultsNoObjects : public TopKResults {
 public:
     TopKResultsNoObjects(int k);
     virtual bool insert(Result result);
};

void planesweep_verbose_on();
void planesweep(Box a, Box b, double lambda,  Results *results);

void subboxes(Box a, Box b, double lambda,  Results *results);

void nlj(Box abox, Box bbox, double lambda,  Results *results);

#endif
