/* Perturb the horizontal cell dataset in order to increase its size
 * and density. */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <sysexits.h>
#include <vlcutils/error.h>
#include <string>
#include <iostream>
#include <getopt.h>
#include "mmapping.h"
#include "ptopk.h"
#include "index.h"

static void usage(FILE *stream)
{
     fprintf(stream ,"Usage %s [-ahv1] INPUT-STEM OUTPUT-STEM\n", 
                  program_name);
     if (stream == stderr)
          fprintf(stream, "Try `%s -h` for more information.\n", program_name);
}

static void help(void)
{
     usage(stdout);
     printf("\n"
"      -h            Display this help.\n"
"      -v            Be verbose.\n"
          );
}

static bool verbose = false;
static std::string input_stem, output_stem;

static void parse_options(int argc, char *argv[])
{
     int opt;

     while ((opt = getopt(argc, argv, "hv")) != -1)
          switch (opt) {
          case 'h': help(); exit(EX_OK);
          case 'v': verbose = true; break;
          default: usage(stderr); exit(EX_USAGE);
          }
     if (argc == optind + 2) {
          input_stem = argv[optind]; 
          output_stem = argv[optind + 1];
     } else {
          usage(stderr);
          exit(EX_USAGE);
     }
         
}

int main(int argc, char *argv[])
{
     set_program_name(argv[0]);
     parse_options(argc, argv);

     ReadableIndex in(input_stem);
     if (verbose) in.info();

     // Block to ensure the index is closed before we print its info.
     {
          WritableIndex out(output_stem, in.npoints(), supersede);

          int lbox = 0;
          for (int oid = 0; oid < in.nobjects; oid++) {
               int k;
               for (k = 0; k < in.nboxes(oid); k++) {
                    Box box = in.get_box(lbox + k);
                    for (int l = 0; l < box.npoints; l++) {
                         Point p = box.get_point(l);
                         out.addPoint(p);
                    }
               }
               lbox += k;
          }
          out.commitObject();
     }
     
     ReadableIndex out(output_stem);
     if (verbose) out.info();
     
     return EX_OK;
}

