#ifndef MMAPPING_H
#define MMAPPING_H

#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <vlcutils/error.h>

template<class T>
class Mmapping {
     int fd;
public:
     size_t length;
     int size;
     const T *data;
     Mmapping(const char *filename_stem, const char *suffix) {
          char *filename;
          if (asprintf(&filename, "%s%s", filename_stem, suffix) == -1)
               fatal_perror("%s", filename_stem);
          fd = open(filename, O_RDONLY);
          if (fd == -1) fatal_perror("%s", filename);
          length = lseek(fd, 0, SEEK_END);
          if (length == (size_t)-1) fatal_perror("lseek");
          if (length % sizeof(T) != 0)
               fatal_error("File size is not divisible by %d: %d",
                           sizeof(T), length);
          size = length / sizeof(T);
          data = (T *) mmap(NULL, length, PROT_READ, 
                      MAP_SHARED, fd, 0);
     }
     ~Mmapping() {
          munmap((void *)data, length);
          close(fd);
     }
};

#endif
