function partition(input_file, output_file, page_size)
% Partition the dataset from a horizontal cell.
% The image is 768 x 512 pixels, so we know that if we divide it into
% 16 x 24 blocks (for a page size of 4096) or 32 x 48 blocks (for a page
% size of 8192), we will get blocks that fit in pages of the desired page.

%page_size = 8192;
point_size = 12;

tuples = load(input_file);

% Find the correct block size
block_size = 256;
while block_size^2 > floor(page_size / point_size)
    block_size = block_size / 2;
end;

tuples = [floor(tuples(:,2) / block_size) floor(tuples(:,3) / block_size) tuples];

% Sort the tuples by block indices.
tuples = sortrows(tuples);

% Iterate through the tuple array and print.
fid = fopen(output_file, 'w');
n = size(tuples, 1);
i = 1;
while (i < n)
    j = i + 1;
    while true
        if j > n || tuples(i,1) ~= tuples(j,1) || tuples(i,2) ~= tuples(i,2)
            k = j - i;
            break;
        end
        j = j + 1;
    end
    fprintf(fid, '%d\n', k);
    for j=i:i+k-1
        fprintf(fid, '%f %f %f\n', tuples(j,3:5));
    end
    i = i + k;
end
fclose(fid);
