/* For a random box, plot its distances to all other boxes. */

#include <stdlib.h>
#include <sysexits.h>
#include <string>
#include <vlcutils/error.h>
#include "mmapping.h"
#include "histogram.h"
#include "ptopk.h"
#include "ddist.h"

struct histogram *allpairs(Mmapping<struct box> &aboxes,
                           Mmapping<struct box> &bboxes)
{
     float *distances = new float[aboxes.size * bboxes.size];
     for (int i = 0; i < aboxes.size; i++) {
          const struct box *abox = &aboxes.data[i];
          for (int j = 0; j < bboxes.size; j++) {
               const struct box *bbox = &bboxes.data[j];
               distances[i * bboxes.size + j] = box_mean_distance(abox, bbox);
          }
     }
     struct histogram *h = histogram(distances, aboxes.size * bboxes.size);
     delete[] distances;
     return h;
}

struct histogram *fromonebox(Mmapping<struct box> &aboxes,
                             Mmapping<struct box> &bboxes)
{
     float *distances = new float[bboxes.size];
     const struct box *abox = &aboxes.data[42];
     for (int j = 0; j < bboxes.size; j++) {
          const struct box *bbox = &bboxes.data[j];
          distances[j] = box_mean_distance(abox, bbox);
     }
     struct histogram *h = histogram(distances, bboxes.size);
     delete[] distances;
     return h;
}

int main(int argc, char *argv[])
{
     set_program_name(argv[0]);
     srandom(time(NULL));
     if (argc != 2) {
          fprintf(stderr, "Usage: %s FACTOR\n", program_name);
          exit(EX_USAGE);
     }
     std::string factor_string = argv[1];
     int factor = atoi(factor_string.c_str());
     if (factor < 1) fatal_error("Factor must be at least one: %s", factor_string.c_str());

     std::string astem = factor_string + "x" + STEM_A;
     std::string bstem = factor_string + "x" + STEM_B;

     Mmapping<struct box> aboxes(astem.c_str(), ".boxes");
     Mmapping<struct box> bboxes(bstem.c_str(), ".boxes");
     struct histogram *h = allpairs(aboxes, bboxes);
//     struct histogram *h = fromonebox(aboxes, bboxes);
     plot_pdf(h);

     return EX_USAGE;
}
