#ifndef PTOPK_H
#define PTOPK_H

#include <assert.h>
#include <vector>

struct point;

class Point {
public:
     float p, x1, x2;
     Point();
     Point(float p, float x1, float x2);
     Point(const struct point *point);
     inline bool operator==(const Point &other) const {
          return p == other.p && x1 == other.x1 && x2 == other.x2;
     }
     bool operator!=(const Point &other) const;
     void write(struct point *external);
     void print();
};

float score(Point point1, Point point2);

class PointSet {
 protected:
     int n;
     std::vector<Point> v;
 public:
     PointSet() : n(0) {}
     virtual int npoints() { return n; }
     virtual Point get_point(int i) {
          return v[i];
     }
     virtual int object_id(int i) { return -1; }
     void add_point(Point p) { v.push_back(p); }
};

struct box;
class BoxSource;
class Results;

// Read-only box used by ReadableIndex
class Box {
protected:
     const struct point *points;
     const BoxSource *index; // Can be NULL.
     int page_id; // Can be -1.
     int my_object_id; // Can be -1.
     int box_size;
 public:
     int npoints;
     float p_min, p_max;
     float x1_min, x1_max;
     float x2_min, x2_max;
     Box(const BoxSource *index, int page_id, const struct box *box_data, 
         const struct point *points);
     Box(int box_size, int object_id, const struct point *points);
     int object_id() const;
     Point get_point(int i);
     double volume() const;
     friend int split(Box &box);
     friend void subboxes1(Box a, Box b, double lambda, Results *results);
     friend void subboxes(Box a, Box b, double lambda, Results *results);
};

class BoxSource {
 public:
     int box_size;
     BoxSource(int box_size) : box_size(box_size) {}
     virtual int object_id(int page_id) const = 0; // { abort(); }
     Box get_box(int page_id);
};

class Reporter {
public:
     virtual void operator()(int object_id_a, Point a, int object_id_b, 
                             Point b) = 0;
};

#define STEM_A "7ca-33-x209y466"
#define STEM_B "7ca-33-x362y289"

#endif
