#include <stdio.h>
#include <math.h>
#include <sysexits.h>
#include <assert.h>
#include <vlcutils/error.h>
#include <vlcutils/mem.h>
#include <vlcutils/pgm.h>
#include <vlcutils/progress.h>

struct pgm8 *read_pgm8_filename(const char *filename)
{
     FILE *f = fopen(filename, "r");
     if (!f) fatal_perror("%s", filename);
     struct pgm8 *im = read_pgm8(f);
     fclose(f);
     return im;
}

double square(double x) { return x * x; }

int main(int argc, char *argv[])
{
     set_program_name(argv[0]);
     if (argc != 3)
          fatal_error("Usage: %s FILENAME-1 FILENAME-2", program_name);
     char *filename1 = argv[1];
     char *filename2 = argv[2];

     struct pgm8 *im1 = read_pgm8_filename(filename1);
     struct pgm8 *im2 = read_pgm8_filename(filename2);
     if (im1->height != im2->height || im1->width != im2->width)
          fatal_error("Image dimension do not agree: %dx%d and %dx%d",
                      im1->height, im1->width, im2->height, im2->width);

     int maxdist = ceil(sqrt(square(im1->height - 1) + 
                             square(im1->width - 1)));
     double *disthist = callock(maxdist, sizeof(double));
     
     struct progress *progress = make_progress(im1->height * im2->width, 
                                               "Computing distances");
     for (int ia = 0; ia < im1->height; ia++)
          for (int ja = 0; ja < im1->width; ja++) {
               double v1 = pgm_pixel(im1, ja, ia);
               if (v1 > 0)
                    for (int ib = 0; ib < im2->height; ib++)
                         for (int jb = 0; jb < im2->width; jb++) {
                              double v2 = pgm_pixel(im2, jb, ib);
                              if (v2 > 0) {
                                   int d = round(sqrt(square(abs(ia - ib)) +
                                                      square(abs(ja - jb))));
                                   assert(d >= 0 && d <= maxdist);
                                   disthist[d] += v1 * v2;
                              }
                         }
               update_progress(progress, ia * im1->width + ja);
          }
     free_progress(progress);

     free(im1);
     free(im2);
     for (int i = 0; i <= maxdist; i++)
          printf("%d %f\n", i, disthist[i]);
     free(disthist);
     return EX_OK;
}
