#include "ptopk.h"
#include "score.h"
#include "index.h"
#include "join.h"

// Returns the index of the first point that is in the second subbox.
int split(Box& box)
{
     struct point *points1 = new struct point[box.box_size];
     struct point *points2 = new struct point[box.box_size];

     double dp = box.p_max - box.p_min, d1 = box.x1_max - box.x1_min;
     double d2 = box.x2_max - box.x2_min;
     int j = 0;
     int k = 0;
     if (dp > d1 && dp > d2) {
          double split = (box.p_min + box.p_max) / 2.0;
          for (int i = 0; i < box.npoints; i++)
               if (box.points[i].p <= split)
                    points1[j++] = box.points[i];
               else
                    points2[k++] = box.points[i];
     } else if (d1 > d2) {
          double split = (box.x1_min + box.x1_max) / 2.0;
          for (int i = 0; i < box.npoints; i++)
               if (box.points[i].x1 <= split)
                    points1[j++] = box.points[i];
               else
                    points2[k++] = box.points[i];
     } else {
          double split = (box.x2_min + box.x2_max) / 2.0;
          for (int i = 0; i < box.npoints; i++)
               if (box.points[i].x2 <= split)
                    points1[j++] = box.points[i];
               else
                    points2[k++] = box.points[i];
     }

     int rv;
     if (j > 0 && k > 0) {
          for (int i = 0; i < j; i++) (struct point)box.points[i] = points1[i];
          for (int i = 0; i < k; i++) (struct point)box.points[j + i] = points2[i];
          for (int i = 0; j + k + i < box.box_size; i++)
               ((struct point *)box.points)[j + k + i].p = -1;
          rv = j;
     } else
          rv = -1;
     delete points1;
     delete points2;
     return rv;
}

void subboxes1(Box a, Box b, double lambda, Results *results)
{
//     printf("subboxes1(%d, %d)\n", a.npoints, b.npoints);
     if (a.npoints < 5 && b.npoints < 5) {
          nlj(a, b, lambda, results);
     } else {
          Score_1D score(lambda);
          if (score.ub(a, b) < results->threshold)
               return;
          if (a.volume() < b.volume())
               subboxes1(b, a, lambda, results);
          else {
               int si = split(a);
               if (si == -1)
                    nlj(a, b, lambda, results);
               else {
                    subboxes1(Box(si, a.object_id(), a.points), 
                              b, lambda, results);
                    subboxes1(Box(a.npoints - si, a.object_id(),
                                  a.points + si), 
                              b, lambda, results);
               }
          }
     }
}

void subboxes(Box a, Box b, double lambda, Results *results)
{
     struct point *pa = new struct point[a.box_size];
     struct point *pb = new struct point[b.box_size];
     memcpy(pa, a.points, a.box_size * sizeof(struct point));
     memcpy(pb, b.points, b.box_size * sizeof(struct point));
     subboxes1(Box(a.box_size, a.object_id(), pa),
               Box(b.box_size, a.object_id(), pb),
               lambda, results);
     delete pa;
     delete pb;
}

